package co.ab180.airbridge.ue;

import android.app.Activity;
import android.app.Application;
import android.content.ContentProvider;
import android.content.ContentValues;
import android.database.Cursor;
import android.net.Uri;
import android.util.Log;

import co.ab180.airbridge.AirbridgeInAppPurchase;
import co.ab180.airbridge.AirbridgeInAppPurchaseEnvironment;
import co.ab180.airbridge.AirbridgeLifecycleIntegration;
import co.ab180.airbridge.OnAttributionResultReceiveListener;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import co.ab180.airbridge.Airbridge;
import co.ab180.airbridge.AirbridgeLogLevel;
import co.ab180.airbridge.AirbridgeOptionBuilder;
import co.ab180.airbridge.OnInAppPurchaseReceiveListener;

public class AirbridgeContentProvider extends ContentProvider {

    @Override
    public boolean onCreate() {
        try {
            Application app = (Application) Objects.requireNonNull(getContext()).getApplicationContext();

            AirbridgeOptionBuilder builder = new AirbridgeOptionBuilder(AirbridgeSettings.appName, AirbridgeSettings.appToken);
            builder.setSDKDevelopmentPlatform("unreal");

            if (isNotEmpty(AirbridgeSettings.sdkSignatureSecretID) && isNotEmpty(AirbridgeSettings.sdkSignatureSecret)) {
                builder.setSDKSignature(AirbridgeSettings.sdkSignatureSecretID, AirbridgeSettings.sdkSignatureSecret);
            }

            builder.setLogLevel(AirbridgeLogLevel.values()[AirbridgeSettings.logLevel]);

            String customDomain = AirbridgeSettings.customDomain;
            if (isNotEmpty(customDomain)) {
                builder.setCustomDomains(joinedStringToList(customDomain, " "));
            }

            builder
                    .setSessionTimeout(AirbridgeSettings.sessionTimeoutSeconds)
                    .setHashUserInformationEnabled(AirbridgeSettings.userInfoHashEnabled)
                    .setCollectLocationEnabled(AirbridgeSettings.locationCollectionEnabled)
                    .setTrackAirbridgeDeeplinkOnlyEnabled(AirbridgeSettings.trackAirbridgeLinkOnly)
                    .setAutoStartTrackingEnabled(AirbridgeSettings.autoStartTrackingEnabled)
                    .setTrackMetaDeferredAppLinkEnabled(AirbridgeSettings.facebookDeferredAppLinkEnabled);

            builder.setOnAttributionReceived(new OnAttributionResultReceiveListener() {
                @Override
                public void onAttributionResultReceived(@NotNull Map<String, String> map) {
                    try {
                        processSetOnAttributionReceived(map);
                    } catch (Throwable e) {
                        Log.e(AirbridgeUnreal.TAG, "Couldn't handle {setOnAttributionReceived}", e);
                    }
                }
            });

            builder
                    .setTrackInSessionLifeCycleEventEnabled(AirbridgeSettings.trackInSessionLifeCycleEventEnabled)
                    .setPauseEventTransmitOnBackgroundEnabled(AirbridgeSettings.pauseEventTransmitOnBackgroundEnabled)
                    .setClearEventBufferOnInitializeEnabled(AirbridgeSettings.clearEventBufferOnInitializeEnabled)
                    .setSDKEnabled(AirbridgeSettings.sdkEnabled);

            String appMarketIdentifier = AirbridgeSettings.appMarketIdentifier;
            if (isNotEmpty(appMarketIdentifier)) {
                builder.setAppMarketIdentifier(appMarketIdentifier);
            }

            builder
                    .setEventBufferCountLimit(AirbridgeSettings.eventBufferCountLimitInGibibyte)
                    .setEventBufferSizeLimit(AirbridgeSettings.eventBufferSizeLimitInGibibyte)
                    .setEventTransmitInterval(AirbridgeSettings.eventTransmitIntervalSeconds);

            String facebookAppId = AirbridgeSettings.facebookAppId;
            if (isNotEmpty(facebookAppId)) {
                builder.setMetaInstallReferrer(facebookAppId);
            }

            builder.setLifecycleIntegration(new AirbridgeLifecycleIntegration() {

                @Nullable
                @Override
                public String getDataString(@NotNull Activity activity) {
                    if (AirbridgeUnreal.airbridgeLifecycleIntegration != null) {
                        return AirbridgeUnreal.airbridgeLifecycleIntegration.getDataString(activity);
                    }
                    return null;
                }
            });

            HashMap<String, String> sdkAttributes = new HashMap<>();
            sdkAttributes.put("wrapperName", "airbridge-unreal-sdk-restricted");
            sdkAttributes.put("wrapperVersion", "4.4.0");
            builder.setSDKAttributes(sdkAttributes);
            
            HashMap<String, Object> sdkWrapperOption = new HashMap<>();
            sdkWrapperOption.put("isHandleAirbridgeDeeplinkOnly", AirbridgeSettings.isHandleAirbridgeDeeplinkOnly);
            builder.setSDKWrapperOption(sdkWrapperOption);
            
            builder.setInAppPurchaseEnvironment(
                    AirbridgeSettings.inAppPurchaseEnvironment.equals(AirbridgeInAppPurchaseEnvironment.SANDBOX.getValue$airbridge_release()) ?
                            AirbridgeInAppPurchaseEnvironment.SANDBOX :
                            AirbridgeInAppPurchaseEnvironment.PRODUCTION
            );
            builder.setOnInAppPurchaseReceived(new OnInAppPurchaseReceiveListener() {
                @Override
                public void onInAppPurchaseReceived(@NotNull AirbridgeInAppPurchase airbridgeInAppPurchase) {
                    try {
                        processOnInAppPurchaseReceived(airbridgeInAppPurchase);
                    } catch (Throwable e) {
                        Log.e(AirbridgeUnreal.TAG, "Couldn't handle {setOnInAppPurchaseReceived}", e);
                    }
                }
            });
            
            builder.setCollectTCFDataEnabled(AirbridgeSettings.collectTCFDataEnabled);
            
            Airbridge.initializeSDK(app, builder.build());
        } catch (Throwable throwable) {
            Log.e(AirbridgeUnreal.TAG, "Couldn't initialize SDK", throwable);
        }
        return true;
    }

    @Override
    public Cursor query(@NotNull Uri uri, String[] projection, String selection, String[] selectionArgs, String sortOrder) {
        return null;
    }

    @Override
    public String getType(@NotNull Uri uri) {
        return null;
    }

    @Override
    public Uri insert(@NotNull Uri uri, ContentValues values) {
        return null;
    }

    @Override
    public int delete(@NotNull Uri uri, String selection, String[] selectionArgs) {
        return 0;
    }

    @Override
    public int update(@NotNull Uri uri, ContentValues values, String selection, String[] selectionArgs) {
        return 0;
    }

    private List<String> joinedStringToList(String joinedString, String delimiter) {
        String[] array = joinedString.split(delimiter);
        List<String> ret = new ArrayList<>();
        for (String element : array) {
            // Remove white space
            if (!element.trim().isEmpty()) {
                ret.add(element);
            }
        }
        return ret;
    }

    private Class<?> getGameActivityClass() throws ClassNotFoundException {
        String[] gameActivityClassNames = {
            "com.epicgames.ue4.GameActivity",           // Unreal Engine 4
            "com.epicgames.unreal.GameActivity",        // Unreal Engine 5
        };
        for (String className : gameActivityClassNames) {
            try {
                return Class.forName(className);
            } catch (Throwable ignore) {
                continue;
            }
        }
        throw new ClassNotFoundException("GameActivity class not found");
    }

    private void processSetOnAttributionReceived(@NotNull Map<String, String> map) throws Exception {
        try {
            getGameActivityClass()
                .getMethod("airbridgeProcessSetOnAttributionReceived", Map.class)
                .invoke(null, map);
        } catch (Throwable e) {
            throw new Exception("Reflection failed", e);
        }
    }

    private void processOnInAppPurchaseReceived(@NotNull AirbridgeInAppPurchase airbridgeInAppPurchase) throws Exception {
        try {
            getGameActivityClass()
                .getMethod("airbridgeProcessOnInAppPurchaseReceived", AirbridgeInAppPurchase.class)
                .invoke(null, airbridgeInAppPurchase);
        } catch (Throwable e) {
            throw new Exception("Reflection failed", e);
        }
    }
    
    private boolean isNotEmpty(String string) {
        if (string == null) { return false; }
        return !(string.isEmpty());
    }
}